from copy import deepcopy
import json


def sanitize(obj):
    res = deepcopy(obj)
    typ = type(res)
    if typ == type(sanitize):
        func_name = res.__name__ or "lambda"
        return "[func {}]".format(func_name)
    elif typ == type({}):
        for i in res.keys():
            res[i] = sanitize(res[i])
    elif typ == type([]):
        fin = []
        for i in res:
            fin.append(sanitize(i))
        res = fin
    elif typ == type(1) or typ == type("1"):
        return str(res)
    else:
        return "[" + str(type(res)) + "]"
    return res



from mw2fcitx.utils import console
import sys
import re

from pypinyin import lazy_pinyin
import opencc
from mw2fcitx.utils import manual_fix


def export(words):
    result = ""
    converter = opencc.OpenCC('t2s.json')
    HANZI_RE = re.compile('^[\u4e00-\u9fa5]+$')
    count = 0
    last_word = None
    for line in words:
        line = line.rstrip("\n")
        if not HANZI_RE.match(line):
            continue

        # Skip single character & too long pages
        if not 1 < len(line):
            continue

        # Skip list pages
        if line.endswith(('列表', '对照表')):
            continue

        if last_word and len(last_word) >= 4 and line.startswith(last_word):
            continue

        pinyin = "'".join(lazy_pinyin(line))
        if pinyin == line:
            # print("Failed to convert, ignoring:", pinyin, file=sys.stderr)
            continue

        if manual_fix(line):
            pinyin = manual_fix(line)
            console.debug(f"Fixing {line} to {pinyin}", file=sys.stderr)

        last_word = line

        result += "\t".join((converter.convert(line), pinyin, "0"))
        result += "\n"
        count += 1
        if count % 1000 == 0:
            console.debug(str(count) + " converted")

    if count % 1000 != 0 or count == 0:
        console.debug(str(count) + " converted")
    return result


from mw2fcitx.pipeline import MWFPipeline
import logging
import sys


def build(config):
    config["source"] = config["source"] or {}
    config["tweaks"] = config["tweaks"] or []
    config["converter"] = config["converter"] or {}
    config["generator"] = config["generator"] or {}
    pipeline = MWFPipeline(config["source"].get("api_path"))
    if config["source"].get("api_path") is None:
        title_file_path = config["source"].get("file_path")
        if title_file_path is None:
            logging.error("No api_path or file_path provided. Stop.")
            sys.exit(1)
        pipeline.load_titles_from_file(title_file_path)
    else:
        pipeline.fetch_titles(**config["source"].get("kwargs"))
    pipeline.convert_to_words(config["tweaks"])
    pipeline.export_words(config["converter"].get("use"),
                          **config["converter"].get("kwargs"))
    pipeline.generate_dict(config["generator"].get("use"),
                           **config["generator"].get("kwargs"))
    return pipeline.dict


from mw2fcitx.tweaks.moegirl import tweaks

exports = {
    "source": {
        "api_path": "https://zh.moegirl.org.cn/api.php",
        "kwargs": {
            "title_limit": 120
        }
    },
    "tweaks": tweaks,
    "converter": {
        "use": "opencc",
        "kwargs": {}
    },
    "generator": {
        "use": "rime",
        "kwargs": {}
    }
}


import sys
import json
from urllib.request import Request, urlopen
from urllib.parse import quote_plus
from mw2fcitx.utils import console
from mw2fcitx.retry import retry


class StatusError(Exception):

    def __init__(self, code):
        super().__init__("HTTP status is {}".format(code))


@retry()
def open_request(url):
    return urlopen(
        Request(
            url,
            headers={
                "User-Agent":
                    "MW2Fcitx/1.0; github.com/outloudvi/fcitx5-pinyin-moegirl"
            }))


def fetch_as_json(url):
    res = open_request(url)
    if res.status == 200:
        return json.loads(res.read())
    else:
        console.error("Error fetching URL {}".format(url))
        raise StatusError(res.status)


def fetch_all_titles(api_url, **kwargs):
    limit = kwargs.get("title_limit") or -1
    console.debug("Fetching titles from {}".format(api_url) +
                  (" with a limit of {}".format(limit) if limit != -1 else ""))
    titles = []
    data = fetch_as_json(api_url + "?action=query&list=allpages&format=json")
    breakNow = False
    while True:
        for i in map(lambda x: x["title"], data["query"]["allpages"]):
            titles.append(i)
            if limit != -1 and len(titles) >= limit:
                breakNow = True
                break
        console.debug("Got {} pages".format(len(titles)))
        if breakNow:
            break
        if "continue" in data:
            data = fetch_as_json(
                api_url +
                "?action=query&list=allpages&format=json&aplimit=max&apcontinue={}"
                .format(quote_plus(data["continue"]["apcontinue"])))
        else:
            break
    console.info("Finished.")
    return titles



from mw2fcitx.pipeline import MWFPipeline


def test_pipeline_basic():
    SNAPSHOT_1 = "测试\tce'shi\t0\n百科\tbai'ke\t0\n"
    pipeline = MWFPipeline()
    pipeline.load_titles(["测试", "百科"])
    pipeline.convert_to_words([])
    pipeline.export_words()
    assert (pipeline.exports == SNAPSHOT_1)
    pipeline.generate_dict(output="test.dict")
    pipeline.generate_dict(generator="rime",
                           output="test.dict.yml",
                           name="test",
                           version="1.0")


if __name__ == "__main__":
    test_pipeline_basic()


import tempfile
import subprocess
from mw2fcitx.utils import console


def gen(text, **kwargs):
    file = tempfile.NamedTemporaryFile("w+")
    file.write(text)
    console.info("Running libime_pinyindict...")
    subprocess.run(["libime_pinyindict", file.name, kwargs["output"]])
    console.info("Dictionary generated.")


from mw2fcitx.fetch import fetch_all_titles
from mw2fcitx.utils import dedup
from mw2fcitx.utils import console
import os
import sys


class MWFPipeline():

    def __init__(self, api_path=""):
        self.api_path = api_path
        self.titles = []
        self.words = []
        self.exports = ""
        self.dict = ""

    def load_titles(self, titles):
        if type(titles) == type(""):
            titles = titles.split("\n")
        self.titles = titles
        console.debug("{} title(s) imported.".format(len(titles)))
        self.words = self.titles

    def load_titles_from_file(self, filename):
        if not os.access(filename, os.R_OK):
            console.error("File {} is not readable".format(filename))
            sys.exit(1)
        self.load_titles(open(filename,"r").read())

    def fetch_titles(self, **kwargs):
        titles = fetch_all_titles(self.api_path, **kwargs)
        self.load_titles(titles)

    def reset_words(self):
        self.words = self.titles

    def convert_to_words(self, pipelines=[]):
        console.debug("Running {} pipelines".format(len(pipelines)))
        cnt = 0
        if type(pipelines) == type(self.convert_to_words):
            pipelines = [pipelines]
        titles = self.titles
        for i in pipelines:
            cnt += 1
            console.debug("Running pipeline {}/{} ({})".format(
                cnt, len(pipelines), i.__name__ or "anonymous function"))
            titles = i(titles)
        self.words = dedup(titles)

    def export_words(self, converter="opencc", **kwargs):
        if converter == "opencc":
            console.debug("Exporting {} words with OpenCC".format(
                len(self.words)))
            from mw2fcitx.exporters.opencc import export
            self.exports = export(self.words, **kwargs)
        elif type(converter) == type(self.export_words):
            console.debug("Exporting {} words with custom converter".format(
                len(self.words)))
            self.exports = converter(self.words, **kwargs)
        else:
            console.error("No such exporter: {}".format(converter))

    def generate_dict(self, generator="pinyin", **kwargs):
        if generator == "pinyin":
            from mw2fcitx.dictgen import pinyin
            dest = kwargs.get("output")
            if not dest:
                console.error(
                    "Dictgen 'pinyin' can only output to files. Please give the file path in the 'output' argument."
                )
                return
            pinyin(self.exports, **kwargs)
        elif generator == "rime":
            from mw2fcitx.dictgen import rime
            self.dict = rime(self.exports, **kwargs)
        else:
            console.error("No such dictgen: {}".format(generator))


# This collation file is for moegirl.org.
# It MIGHT NOT be fit for other wikis.
from mw2fcitx.utils import normalize


def dont_have(string: str, array: [str]):
    for i in array:
        if string.find(i) != -1:
            return False
    return True


def split_and_merge_single(group: [str], spliter: str):
    ret = []
    for i in group:
        for j in i.split(spliter):
            ret.append(j)
    return ret


def tweak_remove_char(char):

    def cb(words):
        return list(map(lambda x: x.replace(char, ""), words))

    return cb


def tweak_len_more_than(length):

    def cb(words):
        return list(filter(lambda x: len(x) > length, words))

    return cb


def tweak_remove_word_includes(items):

    def cb(words):
        return list(filter(lambda x: dont_have(x, items), words))

    return cb


def tweak_split_word_with(spliters):

    def cb(items: [str]):
        ret = items
        for i in spliters:
            tmp = []
            for j in split_and_merge_single(ret, i):
                tmp.append(j)
            ret = tmp
        return ret

    return cb


def tweak_normalize(words):
    ret = []
    for i in words:
        ret.append(normalize(i))
    return ret


tweaks = [
    tweak_remove_word_includes(["○", "〇"]),
    tweak_split_word_with(
        [":", "/", "(", ")", "（", "）", "【", "】", "『", "』", "／"]),
    tweak_len_more_than(1),
    tweak_remove_char("·"), tweak_normalize
]



#!/usr/bin/env python
# coding: utf-8

# Modified based on https://github.com/ytdl-org/youtube-dl/blob/master/setup.py

import os.path
import warnings
import sys
from setuptools import setup, Command, find_packages

exec(compile(open('mw2fcitx/version.py').read(), 'mw2fcitx/version.py', 'exec'))

DESCRIPTION = 'Build fcitx5 libraries from MediaWiki sites'

setup(name='mw2fcitx',
      version=__version__,
      description=DESCRIPTION,
      long_description=open("README.md", encoding="utf-8").read(),
      long_description_content_type='text/markdown',
      url='https://github.com/outloudvi/fcitx5-pinyin-moegirl',
      author='Outvi V',
      author_email='oss@outv.im',
      license='Unlicense',
      include_package_data=True,
      packages=find_packages(),
      install_requires=[
          "OpenCC>=1.1.1.post1",
          "pypinyin>=0.38.1",
      ],
      entry_points={'console_scripts': ['mw2fcitx = mw2fcitx.main:main',]},
      classifiers=[
          'Topic :: Software Development :: Libraries :: Python Modules',
          'Development Status :: 4 - Beta', 'Environment :: Console',
          'License :: Public Domain', 'Programming Language :: Python',
          'Programming Language :: Python :: 3',
          'Programming Language :: Python :: 3.6',
          'Programming Language :: Python :: 3.7',
          'Programming Language :: Python :: 3.8',
          'Programming Language :: Python :: Implementation',
          'Programming Language :: Python :: Implementation :: CPython'
      ])



from optparse import OptionParser
from importlib import import_module
from mw2fcitx.build_dict import build
from mw2fcitx.utils import console, sanitize
import json
import os
import re
import sys


def get_args():
    parser = OptionParser(
        usage="Fetch titles from online and generate a dictionary.")
    parser.add_option("-c",
                      "--config",
                      dest="config",
                      default="config.py",
                      help="configuration file location")
    parser.add_option("-n",
                      "--name",
                      dest="name",
                      default="exports",
                      help="configuration object name")

    return parser.parse_args()


def try_file(file):
    console.debug("Finding config file: {}".format(file))
    if not os.access(file, os.R_OK):
        return False
    file_realpath = os.path.realpath(file)
    console.debug("Config file path: {}".format(file_realpath))
    file_path = os.path.dirname(file_realpath)
    file_name = os.path.basename(file_realpath)
    module_name = re.sub(r"\.py$", "", file_name)
    config_file = False
    try:
        sys.path.insert(1, file_path)
        config_file = import_module(module_name)
    except Exception:
        return False
    finally:
        sys.path.remove(file_path)
    return config_file


def main():
    (options, _) = get_args()
    file = options.config
    objname = options.name
    if file.endswith(".py"):
        config_base = try_file(file)
        if config_base == False:
            # I don't think it works... but let's put it here
            config_base = try_file(file + ".py")
    else:
        config_base = try_file(file + ".py")
    if config_base == False:
        console.error("Config file {} not found or not readable".format(
            "{}, {}.py".format(file, file) if file.endswith("py") else file))
        sys.exit(1)
    console.debug("Parsing config file: {}".format(file))
    if objname not in dir(config_base):
        console.error(
            "Exports not found. Please make sure your config in in a object called '{}'."
            .format(objname))
        sys.exit(1)
    config_object = getattr(config_base, objname)
    console.debug("Config load:")
    displayable_config_object = sanitize(config_object)
    console.debug(
        json.dumps(displayable_config_object, indent=2, sort_keys=True))
    build(config_object)


if __name__ == "__main__":
    main()


import json
from .logger import console
import sys
from os import path

MANUAL_FIX_TABLE = {}
JSON_PATH = path.join(path.dirname(path.realpath(__file__)), "manual_fix.json")

try:
    MANUAL_FIX_TABLE = json.loads(open(JSON_PATH).read())
    console.debug(f"Manual fix table loaded.")
except Exception as e:
    console.debug(f"Error loading manual fix table: {str(e)}")


def manual_fix(text):
    if text in MANUAL_FIX_TABLE:
        return MANUAL_FIX_TABLE[text]
    return None



from mw2fcitx.pipeline import MWFPipeline
from mw2fcitx.tweaks.moegirl import tweaks as moegirl_tweaks


def test_pipeline_basic():
    pipeline = MWFPipeline("https://zh.moegirl.org.cn/api.php")
    pipeline.fetch_titles(limit=50)
    pipeline.convert_to_words(moegirl_tweaks)
    pipeline.export_words()
    pipeline.generate_dict("rime")


if __name__ == "__main__":
    test_pipeline_basic()